"""
    Copyright (C) 2017-2018 Team Kodi
    This file is part of Kodi - kodi.tv

    SPDX-License-Identifier: GPL-3.0-only
    See LICENSES/README.md for more information.
"""

import requests
import urllib3

from .record import WARNING, Record
from .report import Report


def check_url(report: Report, parsed_xml):
    """
     Checks for valid forum, source and website urls
        :parsed_xml: parsed addon.xml file
    """

    # ElementTree doesn't support the following or syntax
    # ./extension[@point='kodi.addon.metadata' or @point='xbmc.addon.metadata']/(forum|source|website)
    sources = parsed_xml.findall("./extension[@point='kodi.addon.metadata']/forum")
    sources += parsed_xml.findall("./extension[@point='xbmc.addon.metadata']/forum")

    sources += parsed_xml.findall("./extension[@point='kodi.addon.metadata']/source")
    sources += parsed_xml.findall("./extension[@point='xbmc.addon.metadata']/source")

    sources += parsed_xml.findall("./extension[@point='kodi.addon.metadata']/website")
    sources += parsed_xml.findall("./extension[@point='xbmc.addon.metadata']/website")

    for source in sources:
        if not source.text:
            continue

        url = source.text
        scheme = True

        try:
            if urllib3.util.parse_url(source.text).scheme is None:
                url = f"http://{source.text}"
                scheme = False

            r = requests.head(url, allow_redirects=True, timeout=5)
            host = urllib3.util.parse_url(r.url).host
            if not scheme and not host.endswith(source.text):
                report.add(Record(WARNING, f"{source.text} redirects to {host}"))
            elif scheme and r.url.rstrip('/') != url.rstrip('/'):
                report.add(Record(WARNING, f"{source.text} redirects to {r.url}"))
            r.raise_for_status()
        except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout, requests.exceptions.HTTPError,
                requests.exceptions.InvalidSchema, requests.exceptions.MissingSchema, requests.exceptions.ReadTimeout,
                requests.exceptions.SSLError, urllib3.exceptions.LocationParseError) as e:
            report.add(Record(WARNING, e))



"""
    Copyright (C) 2021 Team Kodi
    This file is part of Kodi - kodi.tv

    SPDX-License-Identifier: GPL-3.0-only
    See LICENSES/README.md for more information.
"""

import re
from .record import PROBLEM, Record
from .report import Report


def version_is_valid(version):
    """Checks if a version is valid

    Args:
        version (str): The version string

    Returns:
        [bool]: If the version is valid
    """
    return bool(re.match(r"^(\d+\.\d+(\.\d+){0,4}([+~\w]+(\.\d+)?)?)$",
                    version))


def check_version(report: Report, parsed_xml):
    """Checks if the version for the addon defined in addon.xml is valid

    Args:
        report (Report): The report object
        parsed_xml (et.Element): The parsed addon.xml
    """
    if "version" not in parsed_xml.attrib.keys():
        report.add(Record(PROBLEM, "Missing version in addon.xml"))
        return

    if not version_is_valid(parsed_xml.attrib["version"]):
        report.add(Record(PROBLEM, f"Invalid version {parsed_xml.attrib['version']} " \
            "in addon.xml. Please use the major.minor.revision (e.g. 1.0.0) format or " \
            "major.minor.revision+localversion_identifier (e.g. 1.0.0+matrix.1)"))



"""
    Copyright (C) 2017-2018 Team Kodi
    This file is part of Kodi - kodi.tv

    SPDX-License-Identifier: GPL-3.0-only
    See LICENSES/README.md for more information.
"""

import logging
import xml.etree.ElementTree as ET

from .addons.Addon import Addon
from .check_dependencies import VERSION_ATTRB
from .record import INFORMATION, PROBLEM, Record, WARNING
from .report import Report
from .versions import AddonVersion, KodiVersion


LOGGER = logging.getLogger(__name__)


def check_for_existing_addon(report: Report, addon: Addon, all_repo_addons: dict, args):
    """Check if addon submitted already exists or not
        :report: the report object
        :addon: the Addon object (contains id and version)
        :all_repo_addons: dictionary return by all_repo_addon() function
        :args: the args object passed to addon-checker
    """
    # addon details
    addon_name = addon.id
    addon_version = addon.version
    addon_details = {'name': addon_name, 'version': addon_version}

    # args
    pr = bool(args.PR)
    kodi_version = KodiVersion(args.branch)

    is_new_addon = True
    for branch, repo in sorted(all_repo_addons.items(), reverse=True):

        # Addon submission must be higher than the versions already available in lower
        # branches to allow users to receive the update (especially if run with --pr)
        if KodiVersion(branch) <= kodi_version and addon_name in repo:
            is_new_addon = False
            _check_version_higher(report, addon_details, branch, repo.find(addon_name).version, pr)

        # Addon submission must be lower than the versions already available in upper repo branches
        # if that branch corresponds to a breaking change (e.g. version of addon in matrix version >
        # version of addon in gotham) since there might be dependencies not abi-backward compatible.
        elif KodiVersion(branch) > kodi_version and addon_name in repo and \
            not _is_xbmcabi_compatible(addon.dependencies, repr(kodi_version), branch):
            is_new_addon = False
            _check_version_lower(report, addon_details, branch, repo.find(addon_name).version, pr)

    if is_new_addon:
        report.add(Record(INFORMATION, "This is a new addon"))


def _get_addon_name(xml_path: str):
    """returns name and version of the addon
        :xml_path: path of the xml file
    """

    tree = ET.parse(xml_path).getroot()
    return (tree.get("id"), tree.get("version"))


def _is_xbmcabi_compatible(dependencies: list, target_branch: str, upper_branch: str):
    """returns true if the target_branch for this addon is backwards compatible with a
        given upper branch. E.g. if there are dependencies on the addon that are ABI incompatible
        with the migration to an upper kodi version (resulting in the addon being incompatible)
        :dependencies: the list of dependencies of the addon
        :target_branch: the branch the addon lives in (or is being PR'd to)
        :upper_branch: an upper branch that also contains an addon with the same addon id
    """
    for dependency in dependencies:
        if dependency.id in VERSION_ATTRB:
            if AddonVersion(VERSION_ATTRB[dependency.id][upper_branch]["min_compatible"]) > \
                AddonVersion(VERSION_ATTRB[dependency.id][target_branch]["min_compatible"]):
                return False
    return True


def _check_version_higher(report: Report, addon_details, branch, repo_addons_version, pr):
    """Check the version in lower branch is lower than the addon version being submitted
       thus allowing for addon updates.

        :addon_details:       a dict containing name and version of the addon {'name': .., 'version': ..}
        :branch:              branch of the addon present in Kodi repository
        :repo_addons_version: version of addon present in Kodi repository
        :pr:                  boolean value indicating whether the check is
                              running on pull request or not
    """
    addon_name = addon_details.get('name')
    addon_version = addon_details.get('version')

    if AddonVersion(addon_version) <= AddonVersion(repo_addons_version) and pr:
        report.add(
            Record(
                PROBLEM,
                f"{addon_name} addon already exists with a higher or equal version: {repo_addons_version} "\
                    f"in {branch} branch. Users in {branch} won't " \
                     "be able to receive the addon update."
            )
        )
    else:
        if pr:
            report.add(Record(INFORMATION,
            f"{addon_name} addon also exists in {branch} branch but " \
                f"with update compatible version: {repo_addons_version}"))


def _check_version_lower(report: Report, addon_details, branch, repo_addons_version, pr):
    """Check the version in upper branch is higher than the addon version being submitted
       thus allowing for kodi migration and inherent addon update.

        :addon_details:       a dict containing name and version of the addon {'name': .., 'version': ..}
        :branch:              branch of the addon present in Kodi repository
        :repo_addons_version: version of addon present in Kodi repository
        :pr:                  boolean value indicating whether the check is
                              running on pull request or not
    """
    addon_name = addon_details.get('name')
    addon_version = addon_details.get('version')

    if AddonVersion(addon_version) > AddonVersion(repo_addons_version) or \
        (AddonVersion(addon_version) == AddonVersion(repo_addons_version) and pr):
        report.add(
            Record(
                PROBLEM if pr else WARNING,
                f"{addon_name} addon already exists with a lower or equal version: " \
                f"{repo_addons_version} in {branch} branch " \
                "and the addon has non forward abi compatible dependencies. Users migrating " \
                f"to kodi version {branch} won't be able to receive the addon update."
            )
        )
    else:
        if pr:
            report.add(Record(INFORMATION, f"{addon_name} addon also exists "\
                f"in {branch} branch but with migration compatible version: {repo_addons_version}"))



"""
    Copyright (C) 2017-2018 Team Kodi
    This file is part of Kodi - kodi.tv

    SPDX-License-Identifier: GPL-3.0-only
    See LICENSES/README.md for more information.
"""

import os

from kodi_addon_checker import check_addon

from .record import INFORMATION, PROBLEM, Record
from .report import Report


def check_repo(repo_path, args, all_repo_addons, config):
    """Perform all the check on a complete repository

        :repo_path: Path of the repo that is to be tested
        :args: argparse object
        :all_repo_addons: a nested list having information
                          about all the repo addons
        :config: Config object
    """
    repo_report = Report(repo_path)
    repo_report.add(Record(INFORMATION, f"Checking repository {repo_path}"))
    toplevel_folders = sorted(next(os.walk(repo_path))[1])

    for addon_folder in toplevel_folders:
        if addon_folder[0] != '.':
            addon_path = os.path.join(repo_path, addon_folder)
            try:
                addon_report = check_addon.start(addon_path, args, all_repo_addons, config)
                repo_report.add(addon_report)
            except Exception as e: # pylint: disable=broad-except
                repo_report.add(Record(PROBLEM, f"Something went wrong. Please see: {e}"))
    return repo_report



"""
    Copyright (C) 2017-2018 Team Kodi
    This file is part of Kodi - kodi.tv

    SPDX-License-Identifier: GPL-3.0-only
    See LICENSES/README.md for more information.
"""

import logging

from .addons.Addon import Addon
from .record import INFORMATION, PROBLEM, WARNING, Record
from .report import Report
from .versions import AddonVersion, KodiVersion


common_ignore_deps = ['xbmc.metadata.scraper.albums', 'xbmc.metadata.scraper.movies',
                      'xbmc.metadata.scraper.musicvideos', 'xbmc.metadata.scraper.tvshows',
                      'xbmc.metadata.scraper.library', 'xbmc.ui.screensaver', 'xbmc.player.musicviz',
                      'xbmc.python.pluginsource', 'xbmc.python.script', 'xbmc.python.weather', 'xbmc.python.lyrics',
                      'xbmc.python.library', 'xbmc.python.module', 'xbmc.subtitle.module', 'kodi.context.item',
                      'kodi.game.controller', 'xbmc.gui.skin', 'xbmc.webinterface', 'xbmc.addon.repository',
                      'xbmc.pvrclient', 'kodi.gameclient', 'kodi.peripheral', 'kodi.resource', 'xbmc.addon.video',
                      'xbmc.addon.audio', 'xbmc.addon.image', 'xbmc.addon.executable', 'kodi.addon.game',
                      'kodi.audioencoder', 'kodi.audiodecoder', 'xbmc.service', 'kodi.resource.images',
                      'kodi.resource.language', 'kodi.resource.uisounds', 'kodi.resource.games',
                      'kodi.resource.font', 'kodi.inputstream', 'kodi.vfs', 'kodi.imagedecoder', 'xbmc.addon',
                      'xbmc.gui', 'xbmc.json', 'xbmc.metadata', 'xbmc.python', 'script.module.pil',
                      'kodi.binary.global.main', 'kodi.binary.global.audioengine', 'kodi.binary.global.filesystem',
                      'kodi.binary.global.general', 'kodi.binary.global.gui', 'kodi.binary.global.main',
                      'kodi.binary.global.network', 'kodi.binary.global.tools', 'kodi.binary.instance.audiodecoder',
                      'kodi.binary.instance.audioencoder', 'kodi.binary.instance.game',
                      'kodi.binary.instance.imagedecoder', 'kodi.binary.instance.inputstream',
                      'kodi.binary.instance.peripheral', 'kodi.binary.instance.pvr', 'kodi.binary.instance.screensaver',
                      'kodi.binary.instance.vfs', 'kodi.binary.instance.videocodec',
                      'kodi.binary.instance.visualization']

extensions = {"kodi.gameclient": "kodi.binary.instance.game",
              "xbmc.gui.skin": "xbmc.gui",
              "kodi.vfs": "kodi.binary.instance.vfs",
              "xbmc.metadata.scraper.albums": "xbmc.metadata",
              "xbmc.metadata.scraper.artists": "xbmc.metadata",
              "xbmc.metadata.scraper.library": "xbmc.metadata",
              "xbmc.metadata.scraper.movies": "xbmc.metadata",
              "xbmc.metadata.scraper.musicvideos": "xbmc.metadata",
              "xbmc.metadata.scraper.tvshows": "xbmc.metadata",
              "xbmc.pvrclient": "kodi.binary.instance.pvr",
              "xbmc.python.library": "xbmc.python",
              "xbmc.python.lyrics": "xbmc.python",
              "xbmc.python.module": "xbmc.python",
              "xbmc.python.pluginsource": "xbmc.python",
              "xbmc.python.script": "xbmc.python",
              "xbmc.python.weather": "xbmc.python",
              "xbmc.ui.screensaver": "xbmc.python",
              "xbmc.webinterface": "xbmc.json",
              }

VERSION_ATTRB = {
    'xbmc.python': {
        'gotham': {'min_compatible': '2.1.0', 'advised': '2.14.0'},
        'helix': {'min_compatible': '2.1.0', 'advised': '2.19.0'},
        'isengard': {'min_compatible': '2.1.0', 'advised': '2.20.0'},
        'jarvis': {'min_compatible': '2.1.0', 'advised': '2.24.0'},
        'krypton': {'min_compatible': '2.1.0', 'advised': '2.25.0'},
        'leia': {'min_compatible': '2.1.0', 'advised': '2.26.0'},
        'matrix': {'min_compatible': '3.0.0', 'advised': '3.0.0'},
        'nexus': {'min_compatible': '3.0.0', 'advised': '3.0.1'},
        'omega': {'min_compatible': '3.0.0', 'advised': '3.0.1'}
    },
    'xbmc.gui': {
        'gotham': {'min_compatible': '5.0.0', 'advised': '5.0.0'},
        'helix': {'min_compatible': '5.3.0', 'advised': '5.3.0'},
        'isengard': {'min_compatible': '5.3.0', 'advised': '5.9.0'},
        'jarvis': {'min_compatible': '5.10.0', 'advised': '5.10.0'},
        'krypton': {'min_compatible': '5.12.0', 'advised': '5.12.0'},
        'leia': {'min_compatible': '5.14.0', 'advised': '5.14.0'},
        'matrix': {'min_compatible': '5.14.0', 'advised': '5.15.0'},
        'nexus': {'min_compatible': '5.15.0', 'advised': '5.16.0'},
        'omega': {'min_compatible': '5.15.0', 'advised': '5.16.0'}
    },
    'xbmc.json': {
        'gotham': {'min_compatible': '6.0.0', 'advised': '5.0.0'},
        'helix': {'min_compatible': '6.0.0', 'advised': '6.20.0'},
        'isengard': {'min_compatible': '6.0.0', 'advised': '6.25.1'},
        'jarvis': {'min_compatible': '6.0.0', 'advised': '6.32.4'},
        'krypton': {'min_compatible': '6.0.0', 'advised': '7.0.0'},
        'leia': {'min_compatible': '6.0.0', 'advised': '9.7.2'},
        'matrix': {'min_compatible': '6.0.0', 'advised': '12.4.0'},
        'nexus': {'min_compatible': '6.0.0', 'advised': '13.0.0'},
        'omega': {'min_compatible': '6.0.0', 'advised': '13.0.0'}
    },
    'xbmc.addon': {
        'gotham': {'min_compatible': '12.0.0', 'advised': '13.0.0'},
        'helix': {'min_compatible': '12.0.0', 'advised': '14.0.0'},
        'isengard': {'min_compatible': '12.0.0', 'advised': '15.0.0'},
        'jarvis': {'min_compatible': '12.0.0', 'advised': '16.0.0'},
        'krypton': {'min_compatible': '12.0.0', 'advised': '17.0.0'},
        'leia': {'min_compatible': '12.0.0', 'advised': '17.9.910'},
        'matrix': {'min_compatible': '12.0.0', 'advised': '19.1.0'},
        'nexus': {'min_compatible': '12.0.0', 'advised': '20.0.0'},
        'omega': {'min_compatible': '12.0.0', 'advised': '20.90.101'}
    },
    'xbmc.metadata': {
        'gotham': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'helix': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'isengard': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'jarvis': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'krypton': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'leia': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'matrix': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'nexus': {'min_compatible': '1.0', 'advised': '2.1.0'},
        'omega': {'min_compatible': '1.0', 'advised': '2.1.0'}
    },
}

LOGGER = logging.getLogger(__name__)


def check_addon_dependencies(report: Report, repo_addons: dict, parsed_xml, args):
    """Check for any new dependencies in addon.xml file and reports them
        :parsed_xml: parsed addon.xml file
        :repo_addons: dictionary having all the addon list of a particular
                        version of kodi
        :args: the args object passed to addon-checker
    """

    addon = Addon(parsed_xml)
    ignore = _get_ignore_list(KodiVersion(args.branch))
    pr = bool(args.PR)

    for dependency in addon.dependencies:
        if dependency.id in ignore and not dependency.optional:
            pass

        elif dependency.id not in repo_addons:
            if dependency.optional:
                record_level = INFORMATION
            elif not pr:
                record_level = WARNING
            else:
                record_level = PROBLEM

            report.add(Record(record_level,
                              f"{'Optional' if dependency.optional else 'Required'} dependency {dependency.id} " \
                                "is not available in current repository"
            ))

        elif dependency.version is None:
            report.add(Record(INFORMATION if dependency.optional else WARNING,
                              f"{'Optional' if dependency.optional else 'Required'} dependency {dependency.id} " \
                                "does not require a minimum version, " \
                                f"available: {repo_addons.find(dependency.id).version}"))

        elif AddonVersion(repo_addons.find(dependency.id).version) < dependency.version:
            report.add(Record(INFORMATION if dependency.optional else PROBLEM,
                              f"Version mismatch for {'optional' if dependency.optional else 'required'} " \
                              f"dependency {dependency.id}, required: {dependency.version}, "\
                              f"Available: {repo_addons.find(dependency.id).version}"))

        if dependency.id in VERSION_ATTRB:
            try:
                version_info = VERSION_ATTRB[dependency.id][args.branch]
                if AddonVersion(version_info["min_compatible"]) > dependency.version:
                    report.add(Record(PROBLEM, f"For {args.branch}, {dependency.id} version must be " \
                                        f"higher than {version_info['min_compatible']}. " \
                                        f"Advised {version_info['advised']}."))
                    continue

                if AddonVersion(version_info["advised"]) != dependency.version:
                    report.add(Record(WARNING, f"For {args.branch} it is advised to set {dependency.id} " \
                        f"version to {version_info['advised']}"))


            except KeyError:
                LOGGER.warning("Misconfiguration in VERSION_ATTRB of check_dependencies")

    _check_extensions(report, parsed_xml, addon)


def check_reverse_dependencies(report: Report, addon: str, branch_name: str, all_repo_addons: dict):
    """Check for orphaned addon i.e the addons that are not requirement of any other addon.

      :addon: addon that is to be checked
      :branch_name: name of the branch on which the
                    checks are being performed
      :all_repo_addons: a nested list having information
                        about all the repo addonst
    """
    addonInRepo = None
    rdepends = []
    rdependsLowerBranch = []
    branchFound = False

    for branch, repo in sorted(all_repo_addons.items()):
        if not branchFound and branch != branch_name:
            for rdepend in repo.rdepends(addon):
                if rdepend not in rdependsLowerBranch:
                    rdependsLowerBranch.append(rdepend)
            continue
        branchFound = True

        addonFind = repo.find(addon)
        if addonFind and addonInRepo and addonFind != addonInRepo:
            break

        addonInRepo = addonFind

        for rdepend in repo.rdepends(addon):
            if rdepend not in rdependsLowerBranch and rdepend not in rdepends:
                rdepends.append(rdepend)
    if addon.startswith("script.module.") and len(rdepends) + len(rdependsLowerBranch) == 0:
        report.add(Record(WARNING, "This module isn't required by any add-on."))

    if rdepends:
        report.add(Record(INFORMATION,
            f"Reverse dependencies: {', '.join(sorted([r.id for r in rdepends]))} ({len(rdepends)})"
        ))

    if rdependsLowerBranch:
        report.add(Record(INFORMATION,
            "Reverse dependencies (in lower branches): " \
            f"{', '.join(sorted([r.id for r in rdependsLowerBranch]))} ({len(rdependsLowerBranch)})"
        ))


def _get_ignore_list(kodi_version: KodiVersion):
    """Generate an dependency ignore list based
       on the branch name
    """

    if kodi_version >= KodiVersion("leia"):
        common_ignore_deps.extend(["script.module.pycryptodome"])

    if kodi_version == KodiVersion("krypton"):
        common_ignore_deps.extend(["inputstream.adaptive", "inputstream.rtmp"])

    return common_ignore_deps


def _check_extensions(report: Report, parsed_xml, addon):
    """Check if required dependency exist with comparison to
       the existing extension points

      :addon: class kodi_addon_checker.addons.Addon.Addon
    """
    deps = [dependency.id for dependency in addon.dependencies]

    for extension in parsed_xml.findall("extension"):
        point = extension.get("point")
        if point in extensions and extensions[point] not in deps:
            report.add(Record(PROBLEM, f"{extensions[point]} dependency is required for "\
                f"{point} extensions"))



"""
    Copyright (C) 2017-2018 Team Kodi
    This file is part of Kodi - kodi.tv

    SPDX-License-Identifier: GPL-3.0-only
    See LICENSES/README.md for more information.
"""

import logging
import os
import xml.etree.ElementTree as ET

from . import (check_allowed_versions, check_artwork, check_dependencies,
               check_entrypoint, check_files, check_addon_branches,
               check_py3_compatibility, check_string, check_url, common,
               handle_files, schema_validation, ValidKodiVersions)
from .addons.Addon import Addon
from .addons.Repository import Repository
from .versions import KodiVersion
from .record import INFORMATION, PROBLEM, Record
from .report import Report

ROOT_URL = "http://mirrors.kodi.tv/addons/{branch}/addons.xml.gz"
LOGGER = logging.getLogger(__name__)


def start(addon_path, args, all_repo_addons, config=None):
    """Starting point of all the checks that
       are to be performed on an addon

       :addon_path: Path to the addon that is to be checked
       :args: argparse object
       :all_repo_addons: a nested list having information
                         about all the repo addons
    """
    addon_id = os.path.basename(os.path.normpath(addon_path))
    addon_report = Report(addon_id)
    LOGGER.info("Checking add-on %s", addon_id)
    addon_report.add(Record(INFORMATION, f"Checking add-on {addon_id}"))

    repo_addons = all_repo_addons[args.branch]
    addon_xml_path = os.path.join(addon_path, "addon.xml")
    try:
        parsed_xml = ET.parse(addon_xml_path).getroot()
    except ET.ParseError:
        addon_report.add(
                Record(PROBLEM, "Invalid XML data in addon.xml"))

    # check if provided version for the addon is allowed
    check_allowed_versions.check_version(addon_report, parsed_xml)
    # Extract common path from addon paths
    # All paths will be printed relative to this path
    common.REL_PATH = os.path.split(addon_path[:-1])[0]
    addon_xml = check_files.check_addon_xml(addon_report, addon_path, parsed_xml, args.allow_folder_id_mismatch)

    if addon_xml is not None:
        check_addon_branches.check_for_existing_addon(addon_report, Addon(parsed_xml), all_repo_addons, args)

        if not addon_xml.findall("*//broken") and \
           not (addon_xml.findall("*//lifecyclestate") and \
                addon_xml.find("*//lifecyclestate").attrib.get("type") == "broken"):
            file_index = handle_files.create_file_index(addon_path)

            schema_validation.schemas(addon_report, parsed_xml, args.branch)

            check_url.check_url(addon_report, parsed_xml)

            check_dependencies.check_addon_dependencies(addon_report, repo_addons, parsed_xml, args)

            check_dependencies.check_reverse_dependencies(addon_report, addon_id, args.branch, all_repo_addons)

            check_files.check_file_permission(addon_report, file_index)

            check_files.check_for_invalid_xml_files(addon_report, file_index)

            check_files.check_for_invalid_json_files(addon_report, file_index)

            check_artwork.check_artwork(addon_report, addon_path, parsed_xml, file_index, KodiVersion(args.branch))

            max_entrypoint_count = config.configs.get(
                "max_entrypoint_count", 15)
            check_entrypoint.check_complex_addon_entrypoint(
                addon_report, addon_path, parsed_xml, max_entrypoint_count)

            check_py3_compatibility.check_py3_compatibility(addon_report, addon_path, KodiVersion(args.branch))

            if config.is_enabled("check_license_file_exists"):
                # check if license file is existing
                handle_files.addon_file_exists(addon_report, addon_path,
                                               r"^LICENSE\.txt|LICENSE\.md|LICENSE$")

            check_string.check_for_legacy_strings_xml(addon_report, addon_path)

            if KodiVersion(args.branch) >= KodiVersion("isengard"):
                check_files.check_for_new_language_directory_structure(addon_report, addon_path)
            else:
                check_files.check_for_new_language_directory_structure(addon_report, addon_path, supported=False)

            check_string.check_for_invalid_strings_po(addon_report, file_index)

            # Kodi 18 Leia + deprecations
            if config.is_enabled("check_kodi_leia_deprecations"):
                check_string.find_blacklisted_strings(addon_report, addon_path,
                                                      ["System.HasModalDialog", "StringCompare", "SubString",
                                                       "IntegerGreaterThan", "ListItem.ChannelNumber",
                                                       "ListItem.SubChannelNumber", "MusicPlayer.ChannelNumber",
                                                       "MusicPlayer.SubChannelNumber", "VideoPlayer.ChannelNumber",
                                                       "VideoPlayer.SubChannelNumber"],
                                                      [], [".py", ".xml"])

            # General blacklist
            check_string.find_blacklisted_strings(addon_report, addon_path, [], [], [])

            check_files.check_file_whitelist(addon_report, file_index, addon_path)
        else:
            addon_report.add(
                Record(INFORMATION, "Addon marked as broken - skipping"))

    return addon_report


def get_all_repo_addons():
    """Returns a nested dictionary of format:
        {'gotham':{'name_of_addon':'version_of_addon'}}
    """

    repo_addons = {}

    for branch in ValidKodiVersions:
        branch_url = ROOT_URL.format(branch=branch)
        repo_addons[branch] = Repository(branch, branch_url)

    return repo_addons
